unit Glut;

interface

(****************************************************************************
*
*                       SciTech MGL Graphics Library
*
*               Copyright (C) 1991-1997 SciTech Software, Inc.
*                            All rights reserved.
*
*  ======================================================================
*       This library is free software; you can use it and/or
*       modify it under the terms of the SciTech MGL Software License.
*
*       This library is distributed in the hope that it will be useful,
*       but WITHOUT ANY WARRANTY; without even the implied warranty of
*       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*       SciTech MGL Software License for more details.
*  ======================================================================
*
* Filename:     $Workfile:   MGLGLUT.PAS  $
* Version:      $Revision:   1.0  $
*
* Language:     Borland Delphi 2.0
* Environment:  Win32
* C Headers:    gl\glut.h
*
* Description:  Main header file for the OpenGL Utility Toolkit library.
*               This minimal implementation of GLUT has been ported to run
*               on top of SciTech MGL's OpenGL implementation in fullscreen
*               and windowed modes. We dont support menus, overlays or
*               input devices other than the keyboard and mouse. This could
*               be extended as time goes by. The primary purpose of this
*               version of GLUT is to provide a simple framework so we
*               can compile and run the OpenGL sample programs, as well
*               as providing a tool to get MGL programmers up and running
*               quickly with OpenGL support.
*
*               Note that we also only support a single window for output
*               as there is only one output display when running fullscreen.
*
* $Date:   03 Oct 1997 20:24:22  $ $Author:   KendallB  $
*
****************************************************************************)

(*
// Copyright (c) Mark J. Kilgard, 1994, 1995, 1996.

// This program is freely distributable without licensing fees  and is
// provided without guarantee or warrantee expressed or  implied. This
// program is -not- in the public domain.

//
// GLUT API revision history:
//
// GLUT_API_VERSION is updated to reflect incompatible GLUT
// API changes (interface changes, semantic changes, deletions,
// or additions).
//
// GLUT_API_VERSION=1  First public release of GLUT.  11/29/94
//
// GLUT_API_VERSION=2  Added support for OpenGL/GLX multisampling,
// extension.  Supports new input devices like tablet, dial and button
// box, and Spaceball.  Easy to query OpenGL extensions.
//
// GLUT_API_VERSION=3  glutMenuStatus added.
*)

uses
  Gl;

type
    int         = Integer;
    Pint        = ^int;
    unsigned    = Cardinal;
    Bool        = ByteBool;
    uchar       = Char;

const
    GLUT_API_VERSION    = 3;

{-- Display mode bit masks -------------------------------------------------}

const
    GLUT_RGB                        = 0 ;
    GLUT_RGBA                       = GLUT_RGB ;
    GLUT_INDEX                      = 1 ;
    GLUT_SINGLE                     = 0 ;
    GLUT_DOUBLE                     = 2 ;
    GLUT_ACCUM                      = 4 ;
    GLUT_ALPHA                      = 8 ;
    GLUT_DEPTH                      = 16 ;
    GLUT_STENCIL                    = 32 ;
    GLUT_MULTISAMPLE                = 128 ;
    GLUT_STEREO                     = 256 ;
    GLUT_LUMINANCE                  = 512 ;

{-- Mouse buttons ----------------------------------------------------------}

    GLUT_LEFT_BUTTON                = 0 ;
    GLUT_MIDDLE_BUTTON              = 1 ;
    GLUT_RIGHT_BUTTON               = 2 ;

{-- Mouse button callback state --------------------------------------------}

    GLUT_DOWN                       = 0 ;
    GLUT_UP                         = 1 ;

{-- Function keys ----------------------------------------------------------}

    GLUT_KEY_F1                     = 1 ;
    GLUT_KEY_F2                     = 2 ;
    GLUT_KEY_F3                     = 3 ;
    GLUT_KEY_F4                     = 4 ;
    GLUT_KEY_F5                     = 5 ;
    GLUT_KEY_F6                     = 6 ;
    GLUT_KEY_F7                     = 7 ;
    GLUT_KEY_F8                     = 8 ;
    GLUT_KEY_F9                     = 9 ;
    GLUT_KEY_F10                    = 10 ;
    GLUT_KEY_F11                    = 11 ;
    GLUT_KEY_F12                    = 12 ;

{-- Directional keys -------------------------------------------------------}

    GLUT_KEY_LEFT                   = 100 ;
    GLUT_KEY_UP                     = 101 ;
    GLUT_KEY_RIGHT                  = 102 ;
    GLUT_KEY_DOWN                   = 103 ;
    GLUT_KEY_PAGE_UP                = 104 ;
    GLUT_KEY_PAGE_DOWN              = 105 ;
    GLUT_KEY_HOME                   = 106 ;
    GLUT_KEY_END                    = 107 ;
    GLUT_KEY_INSERT                 = 108 ;

{-- Entry/Exit callback state ----------------------------------------------}

    GLUT_LEFT                       = 0 ;
    GLUT_ENTERED                    = 1 ;

{-- Menu usage callback state ----------------------------------------------}

    GLUT_MENU_NOT_IN_USE            = 0 ;
    GLUT_MENU_IN_USE                = 1 ;

{-- Visibility callback state ----------------------------------------------}

    GLUT_NOT_VISIBLE                = 0 ;
    GLUT_VISIBLE                    = 1 ;

{-- Color index component selection values ---------------------------------}

    GLUT_RED                        = 0 ;
    GLUT_GREEN                      = 1 ;
    GLUT_BLUE                       = 2 ;

{-- Layers for use ---------------------------------------------------------}

    GLUT_NORMAL                     = 0 ;
    GLUT_OVERLAY                    = 1 ;

{-- Stroke font opaque addresses (use constants instead in source code) ----}

function    glutStrokeRoman             : Pointer; asmname 'glutStrokeRoman';
function    glutStrokeMonoRoman         : Pointer; asmname 'glutStrokeMonoRoman';

{-- Stroke font constants (use these in GLUT programs) ---------------------}

function    GLUT_STROKE_ROMAN           : Pointer; asmname 'GLUT_STROKE_ROMAN';
function    GLUT_STROKE_MONO_ROMAN      : Pointer; asmname 'GLUT_STROKE_MONO_ROMAN';

{-- Bitmap font opaque addresses (use constants instead in source code) ----}

function    glutBitmap9By15             : Pointer; asmname 'glutBitmap9By15';
function    glutBitmap8By13             : Pointer; asmname 'glutBitmap8By13';
function    glutBitmapTimesRoman10      : Pointer; asmname 'glutBitmapTimesRoman10';
function    glutBitmapTimesRoman24      : Pointer; asmname 'glutBitmapTimesRoman24';
function    glutBitmapHelvetica10       : Pointer; asmname 'glutBitmapHelvetica10';
function    glutBitmapHelvetica12       : Pointer; asmname 'glutBitmapHelvetica12';
function    glutBitmapHelvetica18       : Pointer; asmname 'glutBitmapHelvetica18';

{-- Bitmap font constants (use these in GLUT program) ----------------------}

function    GLUT_BITMAP_9_BY_15         : Pointer; asmname 'GLUT_BITMAP_9_BY_15';
function    GLUT_BITMAP_8_BY_13         : Pointer; asmname 'GLUT_BITMAP_8_BY_13';
function    GLUT_BITMAP_TIMES_ROMAN_10  : Pointer; asmname 'GLUT_BITMAP_TIMES_ROMAN_10';
function    GLUT_BITMAP_TIMES_ROMAN_24  : Pointer; asmname 'GLUT_BITMAP_TIMES_ROMAN_24';
function    GLUT_BITMAP_HELVETICA_10    : Pointer; asmname 'GLUT_BITMAP_HELVETICA_10';
function    GLUT_BITMAP_HELVETICA_12    : Pointer; asmname 'GLUT_BITMAP_HELVETICA_12';
function    GLUT_BITMAP_HELVETICA_18    : Pointer; asmname 'GLUT_BITMAP_HELVETICA_18';

{-- glutGet parameters -----------------------------------------------------}

const
    GLUT_WINDOW_X                   = 100 ;
    GLUT_WINDOW_Y                   = 101 ;
    GLUT_WINDOW_WIDTH               = 102 ;
    GLUT_WINDOW_HEIGHT              = 103 ;
    GLUT_WINDOW_BUFFER_SIZE         = 104 ;
    GLUT_WINDOW_STENCIL_SIZE        = 105 ;
    GLUT_WINDOW_DEPTH_SIZE          = 106 ;
    GLUT_WINDOW_RED_SIZE            = 107 ;
    GLUT_WINDOW_GREEN_SIZE          = 108 ;
    GLUT_WINDOW_BLUE_SIZE           = 109 ;
    GLUT_WINDOW_ALPHA_SIZE          = 110 ;
    GLUT_WINDOW_ACCUM_RED_SIZE      = 111 ;
    GLUT_WINDOW_ACCUM_GREEN_SIZE    = 112 ;
    GLUT_WINDOW_ACCUM_BLUE_SIZE     = 113 ;
    GLUT_WINDOW_ACCUM_ALPHA_SIZE    = 114 ;
    GLUT_WINDOW_DOUBLEBUFFER        = 115 ;
    GLUT_WINDOW_RGBA                = 116 ;
    GLUT_WINDOW_PARENT              = 117 ;
    GLUT_WINDOW_NUM_CHILDREN        = 118 ;
    GLUT_WINDOW_COLORMAP_SIZE       = 119 ;
    GLUT_WINDOW_NUM_SAMPLES         = 120 ;
    GLUT_WINDOW_STEREO              = 121 ;
    GLUT_WINDOW_CURSOR              = 122 ;
    GLUT_SCREEN_WIDTH               = 200 ;
    GLUT_SCREEN_HEIGHT              = 201 ;
    GLUT_SCREEN_WIDTH_MM            = 202 ;
    GLUT_SCREEN_HEIGHT_MM           = 203 ;
    GLUT_MENU_NUM_ITEMS             = 300 ;
    GLUT_DISPLAY_MODE_POSSIBLE      = 400 ;
    GLUT_INIT_WINDOW_X              = 500 ;
    GLUT_INIT_WINDOW_Y              = 501 ;
    GLUT_INIT_WINDOW_WIDTH          = 502 ;
    GLUT_INIT_WINDOW_HEIGHT         = 503 ;
    GLUT_INIT_DISPLAY_MODE          = 504 ;
    GLUT_ELAPSED_TIME               = 700 ;

{-- glutDeviceGet parameters -----------------------------------------------}

    GLUT_HAS_KEYBOARD               = 600 ;
    GLUT_HAS_MOUSE                  = 601 ;
    GLUT_HAS_SPACEBALL              = 602 ;
    GLUT_HAS_DIAL_AND_BUTTON_BOX    = 603 ;
    GLUT_HAS_TABLET                 = 604 ;
    GLUT_NUM_MOUSE_BUTTONS          = 605 ;
    GLUT_NUM_SPACEBALL_BUTTONS      = 606 ;
    GLUT_NUM_BUTTON_BOX_BUTTONS     = 607 ;
    GLUT_NUM_DIALS                  = 608 ;
    GLUT_NUM_TABLET_BUTTONS         = 609 ;

{-- glutLayerGet parameters ------------------------------------------------}

    GLUT_OVERLAY_POSSIBLE           = 800 ;
    GLUT_LAYER_IN_USE               = 801 ;
    GLUT_HAS_OVERLAY                = 802 ;
    GLUT_TRANSPARENT_INDEX          = 803 ;
    GLUT_NORMAL_DAMAGED             = 804 ;
    GLUT_OVERLAY_DAMAGED            = 805 ;

{-- glutUseLayer -----------------------------------------------------------}

{     GLUT_NORMAL                     = 0 ; }
{     GLUT_OVERLAY                    = 1 ; }

{-- glutGetModifiers return mask -------------------------------------------}

    GLUT_ACTIVE_SHIFT               = 1 ;
    GLUT_ACTIVE_CTRL                = 2 ;
    GLUT_ACTIVE_ALT                 = 4 ;

{-- glutSetCursor parameters -----------------------------------------------}

    {  Basic arrows }

    GLUT_CURSOR_RIGHT_ARROW         = 0 ;
    GLUT_CURSOR_LEFT_ARROW          = 1 ;

    {  Symbolic cursor shapees }

    GLUT_CURSOR_INFO                = 2 ;
    GLUT_CURSOR_DESTROY             = 3 ;
    GLUT_CURSOR_HELP                = 4 ;
    GLUT_CURSOR_CYCLE               = 5 ;
    GLUT_CURSOR_SPRAY               = 6 ;
    GLUT_CURSOR_WAIT                = 7 ;
    GLUT_CURSOR_TEXT                = 8 ;
    GLUT_CURSOR_CROSSHAIR           = 9 ;

    {  Directional cursors }

    GLUT_CURSOR_UP_DOWN             = 10 ;
    GLUT_CURSOR_LEFT_RIGHT          = 11 ;

    {  Sizing cursors }

    GLUT_CURSOR_TOP_SIDE            = 12 ;
    GLUT_CURSOR_BOTTOM_SIDE         = 13 ;
    GLUT_CURSOR_LEFT_SIDE           = 14 ;
    GLUT_CURSOR_RIGHT_SIDE          = 15 ;
    GLUT_CURSOR_TOP_LEFT_CORNER     = 16 ;
    GLUT_CURSOR_TOP_RIGHT_CORNER    = 17 ;
    GLUT_CURSOR_BOTTOM_RIGHT_CORNER = 18 ;
    GLUT_CURSOR_BOTTOM_LEFT_CORNER  = 19 ;

    {  Inherit from parent window }

    GLUT_CURSOR_INHERIT             = 100 ;

    {  Blank cursor }

    GLUT_CURSOR_NONE                = 101 ;

    {  Fullscreen crosshair (if available) }

    GLUT_CURSOR_FULL_CROSSHAIR      = 102 ;

{-- GLUT MGL API extensions ------------------------------------------------}

    GLUT_MGL_WINDOWED               = -1 ;
    GLUT_MGL_WINDOWED_320x240       = -2 ;
    GLUT_MGL_WINDOWED_512x384       = -3 ;
    GLUT_MGL_WINDOWED_640x480       = -4 ;
    GLUT_MGL_WINDOWED_800x600       = -5 ;

{-- GLUTtimeval ------------------------------------------------------------}

{ IMHO we dont need this in GPC
type
    GLUTtimeval = record
        tv_sec  : long;
        tv_usec : long;
    end;
}

type
    PPChar  = ^PChar;

{-- GLUT initialization sub-API --------------------------------------------}

{
procedure   glutInit(argcp: pint; argv: PPChar); asmname 'glutInit';
}
procedure   InitGlut; { compatibel to the Delphi-glut }
procedure   glutInit; { alternativ looks more like the C-function }
procedure   glutInitDisplayMode(mode: unsigned); asmname 'glutInitDisplayMode';
procedure   glutInitWindowPosition(x: int; y: int); asmname 'glutInitWindowPosition';
procedure   glutInitWindowSize(width: int; height: int); asmname 'glutInitWindowSize';
procedure   glutMainLoop; asmname 'glutMainLoop';

{-- GLUT MGL extensions sub-API --------------------------------------------}

procedure   glutInitMGLOpenGLType(t_ype: int); asmname 'glutInitMGLOpenGLType';
procedure   glutInitMGLSysPalNoStatic(flag: bool); asmname 'glutInitMGLSysPalNoStatic';
procedure   glutInitMGLDrivers(useLinear, useDirectDraw, useWinDirect: bool); asmname 'glutInitMGLDrivers';
procedure   glutInitMGLPath(MGLPath: PChar); asmname 'glutInitMGLPath';
procedure   glutInitMGLMode(mode: int); asmname 'glutInitMGLMode';
procedure   glutExit; asmname 'glutExit';

{-- GLUT window sub-api ----------------------------------------------------}

function    glutCreateWindow(title: PChar): int; asmname 'glutCreateWindow';
function    glutCreateSubWindow(win: int; x, y, width, height: int): int; asmname 'glutCreateSubWindow';
procedure   glutDestroyWindow(win: int); asmname 'glutDestroyWindow';
procedure   glutPostRedisplay; asmname 'glutPostRedisplay';
procedure   glutSwapBuffers; asmname 'glutSwapBuffers';
function    glutGetWindow: int; asmname 'glutGetWindow';
procedure   glutSetWindow(win: int); asmname 'glutSetWindow';
procedure   glutSetWindowTitle(title: PChar); asmname 'glutSetWindowTitle';
procedure   glutSetIconTitle(title: PChar); asmname 'glutSetIconTitle';
procedure   glutPositionWindow(x, y: int); asmname 'glutPositionWindow';
procedure   glutReshapeWindow(width, height: int); asmname 'glutReshapeWindow';
procedure   glutPopWindow; asmname 'glutPopWindow';
procedure   glutPushWindow; asmname 'glutPushWindow';
procedure   glutIconifyWindow; asmname 'glutIconifyWindow';
procedure   glutShowWindow; asmname 'glutShowWindow';
procedure   glutHideWindow; asmname 'glutHideWindow';
procedure   glutFullScreen; asmname 'glutFullScreen';
procedure   glutSetCursor(cursor: int); asmname 'glutSetCursor';

{-- GLUT overlay sub-API ---------------------------------------------------}

procedure   glutEstablishOverlay; asmname 'glutEstablishOverlay';
procedure   glutRemoveOverlay; asmname 'glutRemoveOverlay';
procedure   glutUseLayer(layer: GLenum); asmname 'glutUseLayer';
procedure   glutPostOverlayRedisplay; asmname 'glutPostOverlayRedisplay';
procedure   glutShowOverlay; asmname 'glutShowOverlay';
procedure   glutHideOverlay; asmname 'glutHideOverlay';

{-- GLUT menu sub-api ------------------------------------------------------}

type
    glutMenuProc    = procedure(i: int);

function    glutCreateMenu(proc: glutMenuProc): int; asmname 'glutCreateMenu';
procedure   glutDestroyMenu(menu: int); asmname 'glutDestroyMenu';
function    glutGetMenu: int; asmname 'glutGetMenu';
procedure   glutSetMenu(menu: int); asmname 'glutSetMenu';
procedure   glutAddMenuEntry(l_abel: PChar; mevalue: int); asmname 'glutAddMenuEntry';
procedure   glutAddSubMenu(l_abel: PChar; submenu: int); asmname 'glutAddSubMenu';
procedure   glutChangeToMenuEntry(item: int; l_abel: PChar; mevalue: int); asmname 'glutChangeToMenuEntry';
procedure   glutChangeToSubMenu(item: int; l_abel: PChar; submenu: int); asmname 'glutChangeToSubMenu';
procedure   glutRemoveMenuItem(item: int); asmname 'glutRemoveMenuItem';
procedure   glutAttachMenu(button: int); asmname 'glutAttachMenu';
procedure   glutDetachMenu(button: int); asmname 'glutDetachMenu';

{-- GLUT callback sub-api --------------------------------------------------}

type
    glutDisplay         = procedure;
    glutReshape         = procedure(width: int; height: int);
    glutKeyboard        = procedure(key: uchar; x, y: int);
    glutMouse           = procedure(button: int; state: int; x, y: int);
    glutMotion          = procedure(x, y: int);
    glutPassiveMotion   = procedure(x, y: int);
    glutEntry           = procedure(state: int);
    glutVisibility      = procedure(state: int);
    glutIdle            = procedure;
    glutTimer           = procedure(tvalue: int);
    glutMenuState       = procedure(state: int);
    glutSpecial         = procedure(key: int; x, y: int);
    glutSpaceballMotion = procedure(x, y, z: int);
    glutSpaceballRotate = procedure(x, y, z: int);
    glutSpaceballButton = procedure(button: int; state: int);
    glutButtonBox       = procedure(button: int; state: int);
    glutDials           = procedure(dial: int; dvalue: int);
    glutTabletMotion    = procedure(x, y: int);
    glutTabletButton    = procedure(button: int; state: int; x, y: int);
    glutMenuStatus      = procedure(status: int; x, y: int);
    glutOverlayDisplay  = procedure;

procedure   glutDisplayFunc(Func: glutDisplay); asmname 'glutDisplayFunc';
procedure   glutReshapeFunc(Func: glutReshape); asmname 'glutReshapeFunc';
procedure   glutKeyboardFunc(Func: glutKeyboard); asmname 'glutKeyboardFunc';
procedure   glutMouseFunc(Func: glutMouse); asmname 'glutMouseFunc';
procedure   glutMotionFunc(Func: glutMotion); asmname 'glutMotionFunc';
procedure   glutPassiveMotionFunc(Func: glutPassiveMotion); asmname 'glutPassiveMotionFunc';
procedure   glutEntryFunc(Func: glutEntry); asmname 'glutEntryFunc';
procedure   glutVisibilityFunc(Func: glutVisibility); asmname 'glutVisibilityFunc';
procedure   glutIdleFunc(Func: glutIdle); asmname 'glutIdleFunc';
procedure   glutTimerFunc(millis: unsigned; Func: glutTimer); asmname 'glutTimerFunc';
procedure   glutMenuStateFunc(Func: glutMenuState); asmname 'glutMenuStateFunc';
procedure   glutSpecialFunc(Func: glutSpecial); asmname 'glutSpecialFunc';
procedure   glutSpaceballMotionFunc(Func: glutSpaceballMotion); asmname 'glutSpaceballMotionFunc';
procedure   glutSpaceballRotateFunc(Func: glutSpaceballRotate); asmname 'glutSpaceballRotateFunc';
procedure   glutSpaceballButtonFunc(Func: glutSpaceballButton); asmname 'glutSpaceballButtonFunc';
procedure   glutButtonBoxFunc(Func: glutButtonBox); asmname 'glutButtonBoxFunc';
procedure   glutDialsFunc(Func: glutDials); asmname 'glutDialsFunc';
procedure   glutTabletMotionFunc(Func: glutTabletMotion); asmname 'glutTabletMotionFunc';
procedure   glutTabletButtonFunc(Func: glutTabletButton); asmname 'glutTabletButtonFunc';
procedure   glutMenuStatusFunc(Func: glutMenuStatus); asmname 'glutMenuStatusFunc';
procedure   glutOverlayDisplayFunc(Func: glutOverlayDisplay); asmname 'glutOverlayDisplayFunc';

{-- GLUT color index sub-api -----------------------------------------------}

procedure   glutSetColor(ndx: int; red, green, blue: GLfloat); asmname 'glutSetColor';
function    glutGetColor(ndx: int; component: int): GLfloat; asmname 'glutGetColor';
procedure   glutCopyColormap(win: int); asmname 'glutCopyColormap';

{-- GLUT state retrieval sub-api -------------------------------------------}

function    glutGet(t_ype: GLenum): int; asmname 'glutGet';
function    glutDeviceGet(t_ype: GLenum): int; asmname 'glutDeviceGet';

{-- GLUT extension support sub-API -----------------------------------------}

function    glutExtensionSupported(name: PChar): int; asmname 'glutExtensionSupported';
function    glutGetModifiers: int; asmname 'glutGetModifiers';
function    glutLayerGet(t_ype: GLenum): int; asmname 'glutLayerGet';

{-- GLUT font sub-api ------------------------------------------------------}

procedure   glutBitmapCharacter(font: Pointer; character: int); asmname 'glutBitmapCharacter';
function    glutBitmapWidth(font: Pointer; character: int): int; asmname 'glutBitmapWidth';
procedure   glutStrokeCharacter(font: Pointer; character: int); asmname 'glutStrokeCharacter';
function    glutStrokeWidth(font: Pointer; character: int): int; asmname 'glutStrokeWidth';

{-- GLUT pre-built models sub-API ------------------------------------------}

procedure   glutWireSphere(radius: GLdouble; slices: GLint; stacks: GLint); asmname 'glutWireSphere';
procedure   glutSolidSphere(radius: GLdouble; slices: GLint; stacks: GLint); asmname 'glutSolidSphere';
procedure   glutWireCone(base: GLdouble; height: GLdouble; slices: GLint; stacks: GLint); asmname 'glutWireCone';
procedure   glutSolidCone(base: GLdouble; height: GLdouble; slices: GLint; stacks: GLint); asmname 'glutSolidCone';
procedure   glutWireCube(size: GLdouble); asmname 'glutWireCube';
procedure   glutSolidCube(size: GLdouble); asmname 'glutSolidCube';
procedure   glutWireTorus(innerRadius: GLdouble; outerRadius: GLdouble; sides: GLint; rings: GLint); asmname 'glutWireTorus';
procedure   glutSolidTorus(innerRadius: GLdouble; outerRadius: GLdouble; sides: GLint; rings: GLint); asmname 'glutSolidTorus';
procedure   glutWireDodecahedron; asmname 'glutWireDodecahedron';
procedure   glutSolidDodecahedron; asmname 'glutSolidDodecahedron';
procedure   glutWireTeapot(size: GLdouble); asmname 'glutWireTeapot';
procedure   glutSolidTeapot(size: GLdouble); asmname 'glutSolidTeapot';
procedure   glutWireOctahedron; asmname 'glutWireOctahedron';
procedure   glutSolidOctahedron; asmname 'glutSolidOctahedron';
procedure   glutWireTetrahedron; asmname 'glutWireTetrahedron';
procedure   glutSolidTetrahedron; asmname 'glutSolidTetrahedron';
procedure   glutWireIcosahedron; asmname 'glutWireIcosahedron';
procedure   glutSolidIcosahedron; asmname 'glutSolidIcosahedron';

{-- Error handling ---------------------------------------------------------}

{ extern void __glutWarning(char *format,...); }
{ extern void __glutFatalError(char *format,...); }
{ extern void __glutFatalUsage(char *format,...); }

{ GRX own functions }

procedure ShowDebug(title: CString); asmname 'ShowDebug';

procedure glutShowBuffer(ctx: Pointer; x1, y1, x2, y2, x3, y3: Integer); asmname 'glutShowBuffer';
procedure GRXSetFrontContext(ctx: Pointer); asmname 'GRXSetFrontContext';

{== Implementation =========================================================}

implementation

procedure   InitGlut; { compatibel to the Delphi-glut }
begin
  glutInit;
end;

procedure   glutInit; { alternativ looks more like the C-function }
{
var
  argc: asmname 'Gpc_argc' Integer;
  argv: asmname 'Gpc_argv' ^CString;

}

procedure   original_glutInit{(argcp: pint; argv: PPChar)}; asmname 'glutInit';

begin
  original_glutInit{(@argc, argv)};
end;


begin
end.

