unit POVUnit;

interface

uses
  GL, Szene_3D;

type
  String255 = String(255);
  Array4f   = array [1..4] of GLfloat;

var
  PovFile: Text;
  wMax, wMin: GlDoubleVector; { fr BoundingBox }
  wStart: Boolean;

function PovTextureName ( MatType: Char; Const MatName: String ) = Result: String255;
function CreatePOVFile (Name:String; r, g, b, ambient: Real): Boolean;
function ClosePOVFile: Boolean;
function WritePOVKommentar (Kommentar: String255): Boolean;
function WritePOVWurstStart (Nummer: Integer): Boolean;
function WritePOVWurstEnde (Material: Integer; Winkel: Real): Boolean;
function WritePOVPolygon (Anzahl: Integer; P: PDynArrVector; Material: Integer): Boolean;
function WritePOVRechteck (a, b, c, d: GLDoubleVector; Material: Integer): Boolean;
function WritePOVDreieck (a, b, c: GLDoubleVector; Material: Integer): Boolean;
function WritePOVLight (L: Array4f; I: Real): Boolean;
function WritePOVCamera (Eye, LookAt: GLDoubleVector; Direction: GLfloat):Boolean;
function WritePOVFlaeche (Anzahl: Integer; P: PDynArrVector; Material: Integer; Kommentar: String255): Boolean;

implementation

uses
  Maths, Treppe3D;

var
  AnzPolys : Integer;


function WritePOVKommentar;
begin
  Writeln(PovFile, '//// ', Kommentar);
  WritePOVKommentar := True;
end;


function PovTextureName ( MatType: Char; Const MatName: String ) = Result: String255;

Var
  j: Integer;

begin (* PovTextureName *)
  if MatName = '' then
    Result:= 'Textur_' + MatType
  else
    begin
      Result:= '';
      for j:= 1 to length ( MatName ) do
        case MatName [ j ] of
          '', '': Result:= Result + 'Ae';
          '', '': Result:= Result + 'Oe';
          '', '': Result:= Result + 'Ue';
          '', '': Result:= Result + 'ae';
          '', '': Result:= Result + 'oe';
          '', '': Result:= Result + 'ue';
          '', '': Result:= Result + 'ss';
          'A'..'Z', 'a'..'z': Result:= Result + MatName [ j ];
        else
          Result:= Result + '_';
        end (* case *);
    end (* else *);
end (* PovTextureName *);


function CreatePOVFile ( Name: String; r, g, b, ambient: Real ): Boolean;

Var
  i: Integer;
  a: Real;

Procedure DeclareTexture ( i: Integer );

Var
  enable: Word;
  MatType: Char;
  MatName, FileName: String ( 255 );
  red, green, blue, ref: Real;

begin (* DeclareTexture *)
  TreppeGetTexture ( i, enable, red, green, blue, MatType, MatName, FileName );
  case MatType of
    'h': ref:= 0.25;
    'm': if enable >= 2 then
           ref:= 0
         else
           ref:= 0.3;
  else
    ref:= 0;
  end (* case *);
  writeln ( PovFile, '#declare ', PovTextureName ( MatType, MatName ), ' = texture {' );
  writeln ( PovFile, '  pigment {' );
  if enable >= 2 then
    begin
      writeln ( PovFile, '    image_map {' );
      writeln ( PovFile, '      tga "', FileName, '"' );
      writeln ( PovFile, '      map_type 0 // 0=planar, 1=spherical, 2=cylindrical, 5=torus interpolate 2 // 0=none, 1=linear, 2=bilinear, 4=normalized distance' );
      writeln ( PovFile, '    } // image_map' );
    end (* if *)
  else
    writeln ( PovFile, '    color rgb <', red : 0 : 2, ',', green : 0 : 2, ',', blue : 0 : 2, '>' );
  writeln ( PovFile, '  } // pigment' );
  writeln ( PovFile, '  finish { reflection ', ref : 0 : 2, ' }' );
  writeln ( PovFile, '  scale 60' );
  writeln ( PovFile, '} // texture' );
end (* DeclareTexture *);

begin
  Name := Name + '.pov';
  Assign(PovFile, Name);
  Rewrite(PovFile);
  Writeln(PovFile, '// Szene aus Treppe3D automatisch generiert');
  Writeln(PovFile);
{  Writeln(PovFile, '#include "mat.inc"  // Treppenmaterialien'); }
  writeln ( PovFile, '// Materialien' );
  writeln ( PovFile, '#include "colors.inc"' );
  writeln ( PovFile, '#include "woods.inc"' );
  writeln ( PovFile, '#include "metals.inc"' );
  writeln ( PovFile, '#include "stones.inc"' );
  writeln ( PovFile );
  for i:= 0 to TreppeGetTextureCount - 1 do
    DeclareTexture ( i );
  writeln ( PovFile );
  writeln ( PovFile, '// Hintergrundfarbe' );
  writeln ( PovFile, 'background { color rgb <', r : 0 : 2, ',',
                     g : 0 : 2, ',', b : 0 : 2, '> }');
  writeln ( PovFile );
  a:= 6 * ambient;
  writeln ( PovFile, '// Hintergrundbeleuchtung' );
  writeln ( PovFile, 'global_settings { ambient_light rgb <', a : 0 : 2, ',',
                     a : 0 : 2, ',', a : 0 : 2, '> }' );
  writeln ( PovFile );
  CreatePOVFile := True;
  AnzPolys      := 0;
end;

function ClosePOVFile;
begin
  Writeln(PovFile, '// Anzahl Polygone: ', AnzPolys:0);
  Writeln(PovFile, '//');
  Writeln(PovFile, '// end. ');
  Writeln(PovFile, '// Szene aus Treppe3D');
  Close(PovFile);
  ClosePOVFile := True;
end;

function RealStr(Zahl:Real):String255;
var
  Zeile : String255;
begin
  WriteStr(Zeile, Zahl:0:5);
  while (Zeile[1] in [' ', '0']) and (Zeile[2] <> '.') do
    Zeile := Zeile[2..length(Zeile)];
  while Zeile[length(Zeile)] = '0' do
    Zeile := Zeile[1..length(Zeile)-1];
  if Zeile[length(Zeile)] = '.' then
    Zeile := Zeile[1..length(Zeile)-1];
  RealStr := Zeile
end;


function WritePOVPolygon (Anzahl: Integer; P: PDynArrVector; Material: Integer): Boolean;
var
  i: Integer;
  enable: Word;
  MatType: Char;
  MatName, FileName: String (255);
  red, green, blue: Real;
  ux, uy, uz, vx, vy, vz, nx, ny, nz, rx, ry, rz, r: Real;
begin
  Write (PovFile, '  polygon {');
  WriteLn (PovFile, '    ', Anzahl + 1 : 0);
  for i := 0 to Anzahl do
     WriteLn (PovFile, '    <',
              RealStr (p^[i].x), ', ',
              RealStr (p^[i].y), ', ',
              RealStr (p^[i].z), '>');
  if Material >= 0 then
    begin
      ux := P^[0].x - P^[1].x;
      uy := P^[0].y - P^[1].y;
      uz := P^[0].z - P^[1].z;
      vx := P^[2].x - P^[1].x;
      vy := P^[2].y - P^[1].y;
      vz := P^[2].z - P^[1].z;
      nx := uy * vz - uz * vy;
      ny := uz * vx - ux * vz;
      nz := ux * vy - uy * vx;
      rx := 0;
      ry := 0;
      rz := 0;
      r := sqrt (sqr (nx) + sqr (ny));
      if r > 0.01 then
        begin
          rx := rphase (nz, r);
          rz := rphase (ny, nx)
        end;
      TreppeGetTexture (Material, enable, red, green, blue, MatType, MatName, FileName);
      WriteLn (PovFile, '  texture { ', PovTextureName (MatType, MatName));
      WriteLn (PovFile, '            rotate <',
               rx * 180 / pi : 0 : 2, ',',
               ry * 180 / pi : 0 : 2, ',',
               rz * 180 / pi : 0 : 2, '> }')
    end;
  WriteLn (PovFile, '  }');
  WritePOVPolygon := true;
  inc (AnzPolys)
end;

function WritePOVRechteck (a, b, c, d: GLDoubleVector; Material: Integer): Boolean;
var
  { p : array[0..4] of GLDoubleVector; }
  p : DynArrVector(4);
begin
  p[0] := a;
  p[1] := b;
  p[2] := c;
  p[3] := d;
  p[4] := a;
  WritePOVRechteck := WritePOVPolygon(4, @p, Material);
end;

function WritePOVDreieck (a, b, c: GLDoubleVector; Material: Integer): Boolean;
var
  p : DynArrVector(3);
begin
  if ((a.x = b.x) and (a.y = b.y) and (a.z = b.z))
  or ((b.x = c.x) and (b.y = c.y) and (b.z = c.z))
  or ((a.x = c.x) and (a.y = c.y) and (a.z = c.z))
  then
    Writeln(PovFile, '  // Dreick degeneriert');

  p[0] := a;
  p[1] := b;
  p[2] := c;
  p[3] := a;
  WritePOVDreieck := WritePOVPolygon(3, @p, Material);
end;

function WritePOVLight;
begin
  Writeln(PovFile, 'light_source { <',RealStr(L[1]), ', ',
                                      RealStr(L[2]), ', ',
                                      RealStr(L[3]),'> color rgb <',
                                      RealStr(2 * I), ', ',
                                      RealStr(2 * I), ', ',
                                      RealStr(2 * I), '>');
  WriteLn(PovFile, '               area_light <100,0,0>, <0,100,0>, 5, 5');
  WriteLn(PovFile, '               adaptive 1 }');
  WritePOVLight := True;
end;

function WritePOVCamera;
begin
  Writeln (PovFile, 'camera { perspective');
  Writeln (PovFile, '  up        <0, 0, 1>');
  Writeln (PovFile, '  right     <1.33, 0, 0>');
  Writeln (PovFile, '  direction <0, 0, ',RealStr(Direction),',>');
  Writeln (PovFile, '  sky       <0, 0, 1>');
  Writeln (PovFile, '  location  <',RealStr(Eye.x), ', ',
                                   RealStr(Eye.y), ', ',
                                   RealStr(Eye.z), '>');
  Writeln (PovFile, '  look_at   <', RealStr(LookAt.x), ', ',
                                    RealStr(LookAt.y), ', ',
                                    RealStr(LookAt.z), '>');
  Writeln (PovFile, '}');
  WritePOVCamera := True;
end;


function WritePOVWurstStart;
begin
  Writeln(PovFile);
  Writeln(PovFile, 'union { // ', Nummer:0);
  WritePOVWurstStart := True;
  wStart := True;
end;


function WritePOVWurstEnde (Material: Integer; Winkel: Real): Boolean;
var
  enable: Word;
  MatType: Char;
  MatName, FileName: String ( 255 );
  red, green, blue: Real;
begin
  TreppeGetTexture ( Material, enable, red, green, blue, MatType, MatName, FileName );
  Writeln(PovFile, '  texture { ', PovTextureName ( MatType, MatName ) );
  if Winkel <> 0 then
    Writeln(PovFile, ' rotate <0,0,', RealStr(Winkel), '> }')
  else
    Writeln(PovFile, ' }');
  Writeln(PovFile, '}');
  WritePOVWurstEnde := True;
end;

function WritePOVFlaeche (Anzahl: Integer; P: PDynArrVector; Material: Integer; Kommentar: String255): Boolean;
begin
  WriteLn (PovFile);
  WriteLn (PovFile, 'union { // ', Kommentar);
  if WritePOVPolygon (Anzahl, P, Material) then;
  WriteLn (PovFile, '}');
  WritePOVFlaeche := true
end;

end.
