unit Spin;

interface

uses
  gl, glut;

procedure SpinSetup;
procedure SpinReshape(width, height: Integer);
procedure SpinIdle;
procedure SpinDraw;

implementation

var
  Xrot, Xstep,
  Yrot, Ystep,
  Zrot, Zstep,
  Step, Scale: GLfloat;
  Obj : GLuint ;
  TestCycles: Integer;

function make_object: GLuint;
var
   list: GLuint;

begin
   list := glGenLists( 1 );

   glNewList( list, GL_COMPILE );

   glBegin( GL_LINE_LOOP );
   glVertex3f(  1.0,  0.5, -0.4 );
   glVertex3f(  1.0, -0.5, -0.4 );
   glVertex3f( -1.0, -0.5, -0.4 );
   glVertex3f( -1.0,  0.5, -0.4 );
   glEnd;

   glBegin( GL_LINE_LOOP );
   glVertex3f(  1.0,  0.5, 0.4 );
   glVertex3f(  1.0, -0.5, 0.4 );
   glVertex3f( -1.0, -0.5, 0.4 );
   glVertex3f( -1.0,  0.5, 0.4 );
   glEnd;

   glBegin( GL_LINES );
   glVertex3f(  1.0,  0.5, -0.4 );   glVertex3f(  1.0,  0.5, 0.4 );
   glVertex3f(  1.0, -0.5, -0.4 );   glVertex3f(  1.0, -0.5, 0.4 );
   glVertex3f( -1.0, -0.5, -0.4 );   glVertex3f( -1.0, -0.5, 0.4 );
   glVertex3f( -1.0,  0.5, -0.4 );   glVertex3f( -1.0,  0.5, 0.4 );
   glEnd;


   glEndList;

   make_object := list;
end;

procedure SpinReshape;
begin
   Writeln('Reshape ', width:5, height:5 );
   glViewport(0, 480 - GLint(height), GLint(width), GLint(height));
   glMatrixMode(GL_PROJECTION);
   glLoadIdentity;
   if height > width then 
     glFrustum(-1, 1, -(height/width), (height/width), 5.0, 15.0 )
   else     
     glFrustum(-(width/height), (width/height), -1, 1, 5.0, 15.0 );
   glMatrixMode(GL_MODELVIEW);
end;

procedure SpinDraw;
begin
   glClear( GL_COLOR_BUFFER_BIT );

   glPushMatrix;

   glTranslatef( 0.0, 0.0, -10.0 );
   glScalef( Scale, Scale, Scale );
   if Xstep <> 0 then
      glRotatef( Xrot, 1.0, 0.0, 0.0 )
   else if Ystep <> 0 then
      glRotatef( Yrot, 0.0, 1.0, 0.0 )
   else
      glRotatef( Zrot, 0.0, 0.0, 1.0 );

   glCallList( Obj );

   glPopMatrix;

   glFlush;
   { Writeln('SpinDraw', TestCycles:10); }
   glutSwapBuffers;
 
end;

procedure IncSteps;
begin
   Xrot := Xrot + Xstep;
   Yrot := Yrot + Ystep;
   Zrot := Zrot + Zstep;
end;

procedure SpinIdle;
begin
   { Writeln('SpinIdle', TestCycles:10); }
   Xrot := Xrot + Xstep;
   Yrot := Yrot + Ystep;
   Zrot := Zrot + Zstep;

   if Xrot >= 360.0 then begin
      Xrot := 0.0;
      Xstep := 0.0;
      Ystep := Step;
   end
   else if Yrot >= 360.0 then begin
      Yrot  := 0.0;
      Ystep := 0.0;
      Zstep := Step;
   end
   else if Zrot >= 360.0 then begin
      Zrot  := 0.0;
      Zstep := 0.0;
      Xstep := Step;
   end;
   { draw; }
   Inc(TestCycles);
   {
   if TestCycles > 10000 then
     halt(0);
   glutPostRedisplay;
   }
end;


Procedure SpinSetup;
begin
   writeln('SpinSetup');
   Step  := 5.0;
   Scale := 1.0;

   Xrot  := 0.0;
   Yrot  := 0.0;
   Zrot  := 0.0;
   Xstep := Step;
   Ystep := 0.0;
   Zstep := 0.0;
   
   TestCycles := 0;

   Obj := make_object;
   glCullFace( GL_BACK );
   glEnable( GL_CULL_FACE );
   (*
   glEnable( GL_DITHER );
   glShadeModel( GL_SMOOTH );
   glEnable( GL_LINE_SMOOTH );
   glEnable( GL_BLEND );
   *)

   glColor3f( 1.0, 1.0, 1.0 );

   (*
   glutReshapeFunc( reshape );
   glutKeyboardFunc( key );
   glutIdleFunc( Idle );
   glutDisplayFunc( Draw );
   *)
end;

end.

