unit szene_3d;

{ Enthlt alle Routinen, die aufgerufen werden mssen/knnen, }
{ um die 3D-Szenerie zu beschreiben }

interface

uses
  Tools, App,
  Gl, Glu;

const
  MaxKanten   = 100;
  MaxPar      = MaxKanten * 3;  { Maximale Anzahl Parameter }
  { z.B. maximal MaxPar/3 Eckpunkte fr eine Wurstscheibe }

  (*
  MaxScheiben = 200;   { Maximale Anzahl Scheiben fr eine Wurst }
  MaxWuerste  = 200;
  *)

type
  Str255 = String[255];
  MyReal = GLDouble;

  GlDoubleVector = record
    x, y, z : GLDouble;
  end;

  { WurstFlaeche            = array[0..MaxKanten] of GlDoubleVector; }
  DynArrVector(n:Integer) = array[0..n] of GlDoubleVector;
  PDynArrVector           = ^DynArrVector;
  DynArrBool(n:Integer)   = array[0..n] of Boolean;

  DreiecksListe(n:Integer) = array[1..n] of DynArrVector(2);
  
  PolyKonkavType = record
    Anzahl   : Integer;
    Ecken    : ^DreiecksListe;
    TexEcken : ^DreiecksListe;
  end;
  
  ScheibenType = record
    Flaechen       : ^DynArrVector;
    Normalen       : ^DynArrVector;
    Dreiecke       : PolyKonkavType;
    Degeneriert    : ^DynArrBool;
    NextScheibe    : ^ScheibenType;
  end;

  { ScheibenArray(n:Integer) = array[0..n] of ScheibenType; }

  WurstType = record
    Material       : Integer;        { Tabellenindex }
    MaterialVektor : GLDoubleVector; { Ausrichtung der Textur }
    MaterialWinkel : MyReal;
    FlaechenForm   : Char;
    BauTeilArt     : Integer; { Dient der Selektion beim Ausblenden }
    BauGruppe      : Integer; { Dient der Selektion beim Ausblenden }
    AnzahlFlaechen : Integer;
    AnzahlKanten   : Integer; { Wieviel Kanten hat die Wurst }
    { Flaechen       : array[0..MaxScheiben, 0..MaxPar] of GlDoubleVector; }
    { Scheiben       : ScheibenArray; }
    VFlaeche       : ^ScheibenType;
    {
    Flaechen       : array[0..MaxScheiben, 0..MaxKanten] of GlDoubleVector;
    Normalen       : array[0..MaxScheiben, 0..MaxKanten] of GlDoubleVector;
    Degeneriert    : array[0..MaxScheiben, 0..MaxKanten] of Boolean;
    }
    nv, nh         : GlDoubleVector; { Flchennormalen der Vorder- und Hinterflche }
    NextWurst      : ^WurstType;
  end;

var
  { Nimmt alle Informationen vom CAD-Programm auf  }

  AktWurst,
  WurstListe    : ^WurstType;

  VorScheibe,
  AktScheibe    : ^ScheibenType;

  NormLen       : MyReal;
  AnzahlDegeneriert,
  AnzahlWuerste : Integer;

  { BlickRichtung/SichtParameter }
  VonWo, NachWo, WoOben: GLDoubleVector;
  BrennWeite, BildBreit, BildHoch : MyReal;

  { Lichtquelle - vielleicht spter mehrere }
  { x, y, z    -  r, g, b }
  Lichtquelle: Array[0..5] of MyReal;

  WurstAktiv            : Boolean;
  AktuellesMaterial     : Integer;
  AktMaterialVektor     : GLDoubleVector;

  {$IFDEF WRITE_PARSE_FEEDBACK }
  o          : Text;
  {$ENDIF }
  Row : Integer; { Wird nur beim Parsen fr Fehlermeldung bentigt }

  (* Fr Fehlermeldungen *)
  ParsenAbgebrochen: Boolean;
  ParseDateiname: pString;

procedure Abbruch(Meldung: Str255);
procedure SetzeVonWo(x, y, z: MyReal);
Procedure SetzeNachWo(x, y, z: MyReal);
Procedure SetzeWoOben(x, y, z: MyReal);
Procedure SetzeBrennWeite(BrennWeite: MyReal);
Procedure SetzeBildGroesse(Breite, Hoehe: MyReal);
Procedure SetzeMaterial(Material: Integer);
Procedure SetzeMaterialVektor(x, y, z: MyReal);
procedure SetzeHintergrund(r, g, b: MyReal);
procedure SetzeLicht(x, y, z, r, g, b: MyReal);
procedure Vorderflaeche(EckZahl: Word; Ecken: Array of MyReal; Form: Char);
procedure Hinterflaeche(EckZahl: Word; Ecken: Array of MyReal; Form: Char);
procedure Mittelflaeche(EckZahl: Word; Ecken: Array of MyReal; Form: Char);
{ function StartShow(var Wuerste: array of WurstType; var Eye, Center, Up: GLDoubleVector): Integer; }
procedure DisposeWurstListe;
procedure WurstTest(var TestWurst: WurstType; Dez: Integer);
function  BerechneFlaechenNormale3v(a,b,c: GLDoubleVector):GLDoubleVector;
function  BerechneFlaechenNormale1f(f: PDynArrVector; EckZahl: Word; Invertieren: Boolean):GLDoubleVector;
procedure WurstInit;
procedure Trianguliere(AnzahlKanten: Integer);

implementation

const Pi     = 3.14159265358979323846;
const halfPi = Pi/2.0;

function Normalize(v: GLDoubleVector): GLDoubleVector;
var l : GLDouble;
begin
  l := sqrt(v.x*v.x + v.y*v.y + v.z*v.z);
  if l > 0.000001 then begin
    v.x := v.x / l;
    v.y := v.y / l;
    v.z := v.z / l;
  end;
  Normalize := v;
end;

function ArcSin (x : MyReal) : MyReal;
begin
  if (x < -1.0) or (x > 1.0) then
    begin
      Abbruch ('ArcSin argument out of range [-1,1]');
      Exit;
    end;
  if x = 1.0 then arcsin := halfPi
  else if x = -1.0 then arcsin := -halfPi
  else begin
    arcsin := ArcTan(Real(x/Sqrt(1.0-Sqr(x))));
  end;
end; { arcsin }

function ArcCos (x : MyReal) : MyReal;
begin
  arccos := halfPi - ArcSin(x);
end; { arccos }

function BerechneWinkel (x, y: MyReal): MyReal;
var
  l, w: MyReal;
begin
  { Normalisieren }
  l := sqrt (sqr(x) + sqr(y));
  if l < 0.0000001 then
    BerechneWinkel := 0
  else
    begin
      x := x / l;
      y := y / l;
      if (x >= 0) then
        w := arccos (y) * 180 / Pi
      else
        w := - arccos (y) * 180 / Pi + 360;
      BerechneWinkel := w
    end
end;

procedure IncAnzahlWuerste;
begin
  Inc(AnzahlWuerste);
end;

procedure WurstTest(var TestWurst: WurstType; Dez: Integer);
begin
  (*
  with TestWurst do begin
    Writeln('WurstTest');
    Writeln('Anzahl Flaechen: ', AnzahlFlaechen:0);
    for j := 0 to AnzahlFlaechen-1 do begin
      if j = 0 then { Vorderflche }
        Writeln('V')
      else if j = AnzahlFlaechen-1 then  { Hinterflche }
        Writeln('H')
      else { Mittelflaeche }
        Writeln('M');
      Writeln('P');
      Writeln(AnzahlKanten:1);
      for i := 0 to AnzahlKanten - 1 do begin
        Writeln(Flaechen[j, i].x:0:Dez);
        Writeln(Flaechen[j, i].y:0:Dez);
        Writeln(Flaechen[j, i].z:0:Dez);
      end;
      { Normalen anzeigen }
      if j = 0 then begin{ Vorderflche }
        Writeln('NV');
        Writeln(nv.x:0:Dez);
        Writeln(nv.y:0:Dez);
        Writeln(nv.z:0:Dez);
      end
      else if j = AnzahlFlaechen-1 then begin { Hinterflche }
        Writeln('NH');
        Writeln(nh.x:0:Dez);
        Writeln(nh.y:0:Dez);
        Writeln(nh.z:0:Dez);
      end;
    end;
  end;
  readln;
  *)
end;

procedure SetzeVonWo(x, y, z: MyReal);
begin
  {$IFDEF WRITE_PARSE_FEEDBACK }
  Writeln(o, 'A');
  Writeln(o, x:0:2);
  Writeln(o, y:0:2);
  Writeln(o, z:0:2);
  {$ENDIF }
end;

Procedure SetzeNachWo(x, y, z: MyReal);
begin
  {$IFDEF WRITE_PARSE_FEEDBACK }
  Writeln(o, 'B');
  Writeln(o, x:0:2);
  Writeln(o, y:0:2);
  Writeln(o, z:0:2);
  {$ENDIF }
end;

Procedure SetzeWoOben(x, y, z: MyReal);
begin
  {$IFDEF WRITE_PARSE_FEEDBACK }
  Writeln(o, 'O');
  Writeln(o, x:0:2);
  Writeln(o, y:0:2);
  Writeln(o, z:0:2);
  {$ENDIF }
end;

Procedure SetzeBrennWeite(BrennWeite: MyReal);
begin
  {$IFDEF WRITE_PARSE_FEEDBACK }
  Writeln(o, 'F');
  Writeln(o, BrennWeite:0:2);
  {$ENDIF }
end;

Procedure SetzeBildGroesse(Breite, Hoehe: MyReal);
begin
  { War wohl falsch spezifiziert }
end;

Procedure SetzeMaterial(Material: Integer);
begin
  AktuellesMaterial := Material;
end;

Procedure SetzeMaterialVektor;
begin
  AktMaterialVektor.x := x;
  AktMaterialVektor.y := y;
  AktMaterialVektor.z := z;
  AktMaterialVektor   := Normalize(AktMaterialVektor);
  {$IFDEF WRITE_PARSE_FEEDBACK }
    Writeln(o, x:0:2);
    Writeln(o, y:0:2);
    Writeln(o, z:0:2);
  {$ENDIF }
end;

procedure SetzeHintergrund(r, g, b: MyReal);
begin
  {$IFDEF WRITE_PARSE_FEEDBACK }
  Writeln(o, 'G');
  Writeln(o, r:0:2);
  Writeln(o, g:0:2);
  Writeln(o, b:0:2);
  {$ENDIF }
end;

procedure SetzeLicht(x, y, z, r, g, b: MyReal);
begin
  {$IFDEF WRITE_PARSE_FEEDBACK }
  Writeln(o, 'L');
  Writeln(o, x:0:2);
  Writeln(o, y:0:2);
  Writeln(o, z:0:2);
  Writeln(o, r:0:2);
  Writeln(o, g:0:2);
  Writeln(o, b:0:2);
  {$ENDIF }
end;

procedure Vorderflaeche(EckZahl: Word; Ecken: Array of MyReal; Form: Char);
var i: Integer;
begin
  if WurstAktiv then
    begin
      Abbruch('Vorderflche vor erwarteter Hinterflche gefunden');
      Exit;
    end (* if *);
  if EckZahl < 3 then
    begin
      Abbruch('Vorderflche hat weniger als 3 Ecken');
      Exit;
    end (* if *);
  {
  if EckZahl > MaxKanten then
    begin
      Abbruch('Vorderflche hat mehr Kanten als in "MaxKanten" vorgesehen');
      Exit;
    end (* if *);
  }
  {$IFDEF WRITE_PARSE_FEEDBACK }
  Writeln(o,'V');
  Writeln(o,'P');
  Writeln(o,EckZahl);
  for i := 1 to EckZahl * 3 do
    Writeln(o, Ecken[i]:0:2);
  {$ENDIF }
  if AnzahlWuerste = 0 then begin
    New(WurstListe);
    AktWurst := WurstListe;
  end else begin
    New(AktWurst^.NextWurst);
    AktWurst := AktWurst^.NextWurst;
  end;
  WurstAktiv := True;
  with AktWurst^ { Wuerste[AnzahlWuerste] } do begin { Mit der aktuellen Wurst arbeiten }
    NextWurst      := NIL;
    { Speicher allokieren }
    New(VFlaeche);
    AktScheibe     := VFlaeche;

    Material       := AktuellesMaterial;
    MaterialWinkel := BerechneWinkel(AktMaterialVektor.x, AktMaterialVektor.y);
    FlaechenForm   := Form;
    BauTeilArt     := 0;
    BauGruppe      := 0;
    AnzahlFlaechen := 0;
    AnzahlKanten   := EckZahl;
    AnzahlFlaechen := 0;
    with AktScheibe^ do begin
      Dreiecke.Anzahl := 0;
      New(Flaechen, EckZahl);
      New(Normalen, EckZahl);
      New(Degeneriert, EckZahl);
      NextScheibe    := NIL;
      for i := 0 to EckZahl - 1 do begin
        Flaechen^[i].x := Ecken[i * 3 + 1];
        Flaechen^[i].y := Ecken[i * 3 + 2];
        Flaechen^[i].z := Ecken[i * 3 + 3];
      end;
      { Die erste Ecke noch einmal hinten anhngen }
      { das vereinfacht den Algo fr die Polygone }
      Flaechen^[AnzahlKanten] := Flaechen^[0];
      {
      nv := BerechneFlaechenNormale(Flaechen[AnzahlFlaechen, 0],
                                    Flaechen[AnzahlFlaechen, 1],
                                    Flaechen[AnzahlFlaechen, 2]);
      }
      {
      for i := 0 to AnzahlKanten-1 do
        Writeln('*',
        Flaechen^[i].x:10:2,
        Flaechen^[i].y:10:2,
        Flaechen^[i].z:10:2);
      }
      nv := BerechneFlaechenNormale1f(Flaechen, AnzahlKanten, False);
    end;
   

    { Hier koennte man noch eine Entscheidung treffen, ob es noetig ist }
    Trianguliere(AnzahlKanten);
    
    Inc(AnzahlFlaechen);
    {
    if AnzahlFlaechen > MaxScheiben then
      begin
        Abbruch('maximale Anzahl fr Wurstflchen berschritten');
        Exit;
      end (* if *);
    }
  end;
end;

procedure Hinterflaeche(EckZahl: Word; Ecken: Array of MyReal; Form: Char);
var i  : Integer;
begin
  if not WurstAktiv then
    begin
      Abbruch('Hinterflche ohne vorherige Vorderflche gefunden');
      Exit;
    end (* if *);
  if EckZahl <> { Wuerste[AnzahlWuerste] } AktWurst^.AnzahlKanten then
    begin
      Abbruch('Anzahl der Ecken von einer Hinterflche stimmt nicht mit Vorderflche berein');
      Exit;
    end (* if *);
  {$IFDEF WRITE_PARSE_FEEDBACK }
  Writeln(o,'H');
  Writeln(o,'P');
  Writeln(o,EckZahl);
  for i := 1 to EckZahl * 3 do
    Writeln(o, Ecken[i]:0:2);
  {$ENDIF }
  with AktWurst^ { Wuerste[AnzahlWuerste] } do begin { Mit der aktuellen Wurst arbeiten }
    { vorhergehede Scheibe fr Zwischenflchenberechnung merken }
    VorScheibe := AktScheibe;
    { Speicher allokieren }
    New(AktScheibe^.NextScheibe);
    AktScheibe   := AktScheibe^.NextScheibe;
    AnzahlKanten := EckZahl;
    {
    Writeln('Hinterflche');
    for i := 1 to EckZahl * 3 do
      Write(Ecken[i]:8:2);
    Writeln;
    }
    with AktScheibe^ do begin
      Dreiecke.Anzahl := 0;
      New(Flaechen, EckZahl);
      { New(Normalen, EckZahl);    }
      { New(Degeneriert, EckZahl); }
      NextScheibe    := NIL;
      for i := 0 to EckZahl - 1 do begin
        Flaechen^[i].x := Ecken[i * 3 + 1];
        Flaechen^[i].y := Ecken[i * 3 + 2];
        Flaechen^[i].z := Ecken[i * 3 + 3];
      end;
      { Die erste Ecke noch einmal hinten anhngen }
      { das vereinfacht den Algo fr die Polygone }
      Flaechen^[AnzahlKanten] := Flaechen^[0];
      for i := 0 to AnzahlKanten-1 do begin
         { Writeln('Normale ',i:0); }
         VorScheibe^.Normalen^[i] :=
            BerechneFlaechenNormale3v(VorScheibe^.Flaechen^[i  ],
                                                  Flaechen^[i  ],
                                                  Flaechen^[i+1]);
         VorScheibe^.Degeneriert^[i] := NormLen < 0.1;
         if NormLen < 0.1 then
           inc(AnzahlDegeneriert);
      end;
      {
      nh := BerechneFlaechenNormale(Flaechen[AnzahlFlaechen, 0],
                                    Flaechen[AnzahlFlaechen, 1],
                                    Flaechen[AnzahlFlaechen, 2]);
      }
      nh := BerechneFlaechenNormale1f(Flaechen, AnzahlKanten, True);
    end;
    
    { Hier koennte man noch eine Entscheidung treffen, ob es noetig ist }
    Trianguliere(AnzahlKanten);
    
    Inc(AnzahlFlaechen);
    {
    if AnzahlFlaechen > MaxScheiben then
      begin
        Abbruch('maximale Anzahl fr Wurstflchen berschritten');
        Exit;
      end (* if *);
    }
  end;
  WurstAktiv    := False;
  IncAnzahlWuerste; { Wieder eine Wurst komplett abgespeichert }
end;

procedure Mittelflaeche(EckZahl: Word; Ecken: Array of MyReal; Form: Char);
var i  : Integer;
begin
  if not WurstAktiv then
    begin
      Abbruch('Mittelflche ohne vorherige Vorderflche gefunden');
      Exit;
    end (* if *);
  if EckZahl <> { Wuerste[AnzahlWuerste] } AktWurst^.AnzahlKanten then
    begin
      Abbruch('Anzahl der Ecken von einer Mittelflche stimmt nicht mit Vorderflche berein');
      Exit;
    end (* if *);
  {$IFDEF WRITE_PARSE_FEEDBACK }
  Writeln(o,'M');
  Writeln(o,'P');
  Writeln(o,EckZahl);
  for i := 1 to EckZahl * 3 do
    Writeln(o, Ecken[i]:0:2);
  {$ENDIF }
  with AktWurst^ { Wuerste[AnzahlWuerste] } do begin { Mit der aktuellen Wurst arbeiten }
    { vorhergehede Scheibe fr Zwischenflchenberechnung merken }
    VorScheibe := AktScheibe;
    { Speicher allokieren }
    New(AktScheibe^.NextScheibe);
    AktScheibe   := AktScheibe^.NextScheibe;
    AnzahlKanten := EckZahl;
    with AktScheibe^ do begin
      Dreiecke.Anzahl := 0;
      New(Flaechen, EckZahl);
      New(Normalen, EckZahl);
      New(Degeneriert, EckZahl);
      NextScheibe    := NIL;
      for i := 0 to EckZahl - 1 do begin
        Flaechen^[i].x := Ecken[i * 3 + 1];
        Flaechen^[i].y := Ecken[i * 3 + 2];
        Flaechen^[i].z := Ecken[i * 3 + 3];
      end;
      { Die erste Ecke noch einmal hinten anhngen }
      { das vereinfacht den Algo fr die Polygone }
      Flaechen^[AnzahlKanten] := Flaechen^[0];
      for i := 0 to AnzahlKanten-1 do begin
         { Writeln('Normale ',i:0); }
         VorScheibe^.Normalen^[i] :=
            BerechneFlaechenNormale3v(VorScheibe^.Flaechen^[i  ],
                                                  Flaechen^[i  ],
                                                  Flaechen^[i+1]);
         VorScheibe^.Degeneriert^[i] := NormLen < 0.1;
         if NormLen < 0.1 then
           inc(AnzahlDegeneriert);
      end;
    end;
    Inc(AnzahlFlaechen);
    {
    if AnzahlFlaechen > MaxScheiben then
      begin
        Abbruch('maximale Anzahl fr Wurstflchen berschritten');
        Exit;
      end (* if *);
    }
  end;
end;

procedure DisposeWurstListe;
var
  TempWurst,
  TempScheibe : Pointer;
  i : Integer;
begin
  AktWurst := WurstListe;
  i := 0;
  while AktWurst <> Nil do begin
{    Writeln(i); }
    inc(i);
    TempWurst := AktWurst^.NextWurst;
    AktScheibe := AktWurst^.VFlaeche;
    while AktScheibe <> Nil do begin
      TempScheibe := AktScheibe^.NextScheibe;
      Dispose(AktScheibe^.Flaechen);
      if AktScheibe^.NextScheibe <> Nil then begin
        { Hinterflche hat keine zugeordneten Seitenflchen }
        Dispose(AktScheibe^.Normalen);
        Dispose(AktScheibe^.Degeneriert);
      end;
      Dispose(AktScheibe);
      AktScheibe := TempScheibe;
    end;
    Dispose(AktWurst^.VFlaeche);
    Dispose(AktWurst);
    AktWurst:= TempWurst;
  end; (* while *);
  WurstListe:= Nil;
end;

procedure Abbruch(Meldung: Str255);
begin
  ParsenAbgebrochen:= true;
 (*$if 0 *)
    {$IFDEF WRITE_PARSE_FEEDBACK }
    writeln('Fehler beim Scannen der 3D-Datei in Zeile ', Row ,' entdeckt:');
    writeln(o, Meldung);
    Close(o);
    {$ENDIF }
    writeln;
    writeln('Fehler beim Scannen der 3D-Datei in Zeile ', Row ,' entdeckt:');
    writeln( Meldung );
    halt(1);
  (*$else *)
    writeln ( StdErr, Meldung );
    if ExecDialog ( 'ReadErr', @ParseDateiName ) = 0 then;
  (*$endif *)
end;

(*

function StartShow(var Wuerste: array of WurstType; var Eye, Center, Up: GLDoubleVector): Integer;
begin
  {
  Wuerste[0] := Wurst;
  Eye := VonWo;
  Center := NachWo;
  Up     := WoOben;
  }
  { Brennweite mu noch bergeben werden }
  StartShow := 0;
end;

*)

function VectSub(a, b: GLDoubleVector): GLDoubleVector;
var r: GLDoubleVector;
begin
  r.x := a.x - b.x;
  r.y := a.y - b.y;
  r.z := a.z - b.z;
  VectSub := r;
end;

function VectMul(a, b: GLDoubleVector): GLDoubleVector;
var r: GLDoubleVector;
begin
  r.x := a.y * b.z - a.z * b.y;
  r.y := a.z * b.x - a.x * b.z;
  r.z := a.x * b.y - a.y * b.x;
  VectMul := r;
end;

function BerechneFlaechenNormale3v(a,b,c: GLDoubleVector):GLDoubleVector;
var r : GLDoubleVector;
    l : GLfloat;
begin
  r := VectMul(VectSub(a,b), VectSub(b,c));
  l := sqrt(r.x*r.x + r.y*r.y + r.z*r.z);
  if l > 0.000001 then begin
    r.x := r.x / l * 3;
    r.y := r.y / l * 3;
    r.z := r.z / l * 3;
  end else begin
    {
    Writeln(a.x:10:2,a.y:10:2,a.z:10:2);
    Writeln(b.x:10:2,b.y:10:2,b.z:10:2);
    Writeln(c.x:10:2,c.y:10:2,c.z:10:2);
    Writeln(r.x:10:2,r.y:10:2,r.z:10:2);
    Abbruch('Bei Berechnung der Flchennormale liegen mehrere Punkte auf einer Linie');
    Exit;
    }
  end;
  NormLen := l; { Lnge merken, um degenerierte Flchen zu markieren zu knnen }
  BerechneFlaechenNormale3v := r;
end;

function  BerechneFlaechenNormale1f(f: PDynArrVector; EckZahl: Word; Invertieren: Boolean):GLDoubleVector;
{ Wird bentigt, da auch konkave Flchen existieren
  und dort versagte leider meine Standardberechnung
  mit den ersten 3 Ecken der Flche}
var
  am, f0, f1, f2, u, v : GLDoubleVector; { Mittelpunkt zwischen f^[a1] und f^[a2] }
  i: Integer;
  l: MyReal;
begin
  { Wenn der letzte gleich dem ersten Eckpunkt ist, verringern wird die Anzahl der Ecken }
  i := 0;
  while (f^[EckZahl - 1].x = f^[i].x) and
        (f^[EckZahl - 1].y = f^[i].y) and
        (f^[EckZahl - 1].z = f^[i].z)
  do begin
    dec(EckZahl);
    if EckZahl < 3 then
      begin
        Abbruch('Eckzahl wurde durch Reduzierung unter 3 gesetzt');
        Exit;
      end (* if *);
    {
    Writeln('Eckzahl reduziert');
    }
  end;

(*$if 1 *)

  Result.x:= 0;
  Result.y:= 0;
  Result.z:= 0;
  for i:= 0 to EckZahl - 1 do
    begin
      f0:= f^ [ i ];
      f1:= f^ [ ( i + 1 ) mod EckZahl ];
      f2:= f^ [ ( i + 2 ) mod EckZahl ];
      u.x:= f1.x - f0.x;
      u.y:= f1.y - f0.y;
      u.z:= f1.z - f0.z;
      v.x:= f2.x - f1.x;
      v.y:= f2.y - f1.y;
      v.z:= f2.z - f1.z;
      am.x:= u.y * v.z - v.y * u.z;
      am.y:= v.x * u.z - u.x * v.z;
      am.z:= u.x * v.y - v.x * u.y;
      Result.x:= Result.x + am.x;
      Result.y:= Result.y + am.y;
      Result.z:= Result.z + am.z;
    end (* for *);
  with Result do
    begin
      l:= sqrt ( sqr ( x ) + sqr ( y ) + sqr ( z ) );
      x:= x / l;
      y:= y / l;
      z:= z / l;
    end (* with *);

(*$else *)

  { Ermittele die zwei am weitesten auseinanderliegenden Ecken }
  l1 := -1;
  {
  for j := 0 to EckZahl - 1 do
    Writeln('#',f^[j].x:10:2,f^[j].y:10:2,f^[j].z:10:2);
  }
  for j := 0 to EckZahl - 1 do
    for i := 0 to EckZahl - 1 do begin
      l := sqrt(sqr(f^[j].x - f^[i].x) + sqr(f^[j].y - f^[i].y) + sqr(f^[j].z - f^[i].z));
      if l > l1 then begin
        l1 := l;
        a1 := j;
        a2 := i;
      end;
    end;

  { Mittelpunkt ermitteln }
  am.x := (f^[a1].x + f^[a2].x) / 2;
  am.y := (f^[a1].y + f^[a2].y) / 2;
  am.z := (f^[a1].z + f^[a2].z) / 2;
  {
  Writeln(am.x:10:2,am.y:10:2,am.z:10:2, a1:5, a2:5);
  }
  { Suche die davon am weitesten entfernte Ecke }
  l2 := -1;
  for i := 0 to EckZahl - 1 do begin
    { Die bereits gefundenen Ecken nicht bercksichtigen }
    if (i <> a1) and (i <> a2) then
      l := sqrt(sqr(am.x - f^[i].x) + sqr(am.y - f^[i].y) + sqr(am.z - f^[i].z));
    if l > l2 then begin
      l2 := l;
      b  := i;
    end;
  end;
  {
  Writeln(f^[b].x:10:2,f^[b].y:10:2,f^[b].z:10:2, b:5);
  }

  { Ecken sortieren a1 <= a2 <= b }
  if a1 > a2 then begin
    i  := a2;
    a2 := a1;
    a1 := i;
  end;
  if a2 > b then begin
    i  := a2;
    a2 := b;
    b := i;
  end;
  if a1 > a2 then begin
    i  := a2;
    a2 := a1;
    a1 := i;
  end;
  {
  Writeln('Sortierungergebnis : ',a1:5, a2:5, b:5);
  }

  { Normale berechnen }
  if Invertieren then
    am := BerechneFlaechenNormale3v(f^[a2], f^[a1], f^[b])
  else
    am := BerechneFlaechenNormale3v(f^[a1], f^[a2], f^[b]);
  BerechneFlaechenNormale1f := am;

(*$endif *)

end;


procedure WurstInit;
begin
  { Initialisierung }
  AktuellesMaterial   := 1;
  AktMaterialVektor.x := 0;
  AktMaterialVektor.y := 0;
  AktMaterialVektor.z := 0;
  WurstAktiv          := False;
  AnzahlDegeneriert   := 0;
  AnzahlWuerste       := 0;
  WurstListe          := NIL;
  AktWurst            := NIL;
end;

var 
  tess  : PGLUtesselator;
  TestT : Boolean;
  AnzD, EckeD : Integer;

procedure glu_tess_begin_callback(err: GLenum);
begin
  if not TestT then begin
    { Happy day!!! Nothing to do ;-) }
  end;    
end;

procedure glu_tess_vertex_callback(pg: Pointer); 
var g: ^GLDoubleVector;
begin
  if not TestT then begin
    g := pg;
    {
    writeln(AnzD:10, EckeD:10, ' Vertex', g^.x:10:3, g^.y:10:3, g^.z:10:3);
    }
    with AktScheibe^ do with Dreiecke do begin
      { Ecke abspeichern }
      Ecken^[AnzD][EckeD] := g^; {x,y,z}
      { Texturkoordinaten berechnen }
      TexEcken^[AnzD][EckeD].x := Ecken^[AnzD][EckeD].x - Flaechen^[0].x;
      TexEcken^[AnzD][EckeD].y := Ecken^[AnzD][EckeD].y - Flaechen^[0].y;
      TexEcken^[AnzD][EckeD].z := Ecken^[AnzD][EckeD].z - Flaechen^[0].z;
    end;
  end;    
  Inc(EckeD);
end;  

procedure glu_tess_end_callback;
begin
  if not TestT then begin
    { Nothing to do ;-) }
  end;    
  { Das Dreieck wurde komplett uebergeben }
  Inc(AnzD);
  EckeD := 0;
end;  

procedure Trianguliere(AnzahlKanten: Integer); { Bezieht sich immer auf AktScheibe^ }
var i: Integer;
begin
{  writeln ( StdErr, '0' ); }
  with AktScheibe^ do begin
{    writeln ( StdErr, '1' ); }
    { Dreiecke zaehlen }
    AnzD  := 0;
    TestT := True;
    
{    writeln ( StdErr, '2' ); }
    gluBeginPolygon(tess);
{    writeln ( StdErr, '2.1: ', AnzahlKanten ); }
    for i := (AnzahlKanten-1) downto 0 do
      gluTessVertex(tess,@Flaechen^[i].x,@Flaechen^[i].x);
{    writeln ( StdErr, '2.2' ); }
    gluEndPolygon(tess);
{    writeln ( StdErr, '3' ); }
  
    Dreiecke.Anzahl := AnzD;
    { Platz fuer Dreiecke schaffen }
    new(Dreiecke.Ecken, AnzD);
    new(Dreiecke.TexEcken, AnzD);
{    writeln ( StdErr, '4' ); }
    
    { Werte durch Callbacks eintragen lassen }
    AnzD  := 1;
    TestT := False;
    
{    writeln ( StdErr, '5' ); }
    gluBeginPolygon(tess);
    for i := (AnzahlKanten-1) downto 0 do
      gluTessVertex(tess,@Flaechen^[i].x,@Flaechen^[i].x);
    gluEndPolygon(tess);
{    writeln ( StdErr, '6' ); }

  end;      
end;      

begin
  { Callback-Routinen fuer Tessalationsobjekt installieren }
  tess := gluNewTess; { Muss man das Ding unbedingt wieder freigeben??? }
  gluTessCallback_begin(tess, GLU_TESS_BEGIN, glu_tess_begin_callback); 
  gluTessCallback_vertex(tess, GLU_TESS_VERTEX, glu_tess_vertex_callback);
  gluTessCallback(tess, GLU_TESS_END, glu_tess_end_callback);
end.
