unit TgaUnit2;

(*$I-*)

Interface

uses
  Objects;

Type
  String255  = String(255);

  tTGAHeader = record
    crap1    : Byte;
    colormap : Byte;
    BildTyp  : Byte;
    crap2    : array[1..3] of Byte;
    Unknown  : ShortInt;
    crap3    : array[1..4] of Byte;
    breite   : ShortInt;
    hoehe    : ShortInt;
    BitsPerPixel: Byte;
    SubType  : Byte;
  end;
  
  TexturArrayT(n:Integer)  = array[1..n] of Byte;

const
  MaxWidth = 1600; { grte Bildbreite zum Sichern }

var
  f : tDosStream;
  d : Text;
  Result : Integer;
  kopf : tTGAHeader;
  kopf2: packed array[1..18] of Byte absolute kopf;
  ParFileName : String(8);
  Comments    : array[1..4] of String(64);
  ColorTable  : array[0..2048, 1..3] of byte;
  zeile       : array[0 .. MaxWidth * 3] of Byte;

function ReadTGAFile(Name:String255; var TexturArray: TexturArrayT; var Breit, Hoch: LongInt): boolean;
function SaveScreen2TGA(Name:String255; var PixelArray: TexturArrayT; w,h:integer): boolean;

implementation

function ReadTGAFile;
var
  i,j,k     : Integer;

begin
  f.Init ( Name, stOpenRead );

  if f.Status <> stOK then begin
    ReadTGAFile := False;
    Exit;
  end;

  { read TGAHeader }
  f.Read ( Kopf2 [ 1 ], 18 );

  if kopf.colormap <> 0 then begin  { keine RGB-Tabelle }
    f.Done;
    ReadTGAFile := False;
    Exit;
  end;

  if kopf.BitsPerPixel <> 24 then begin  { kein RGB-Bild }
    f.Done;
    ReadTGAFile := False;
    Exit;
  end;

  { Bild passt nicht in das vorgesehene Array }
  if (Breit < kopf.Breite) or (Hoch < kopf.Hoehe)
  or (Breit * Hoch = 0) then begin
    { korrekte Werte zurckliefern }
    Breit := kopf.Breite;
    Hoch  := kopf.Hoehe;
    f.Done;
    ReadTGAFile := False;
    Exit;
  end;

  Breit := kopf.Breite;
  Hoch  := kopf.Hoehe;

  (* Kommentar berlesen *)
  if kopf.crap1 > 0 then
    f.Read ( zeile [ 0 ], kopf.crap1 );

  if kopf.SubType = $20 then { Top - Down }
    for i := 0 to kopf.Hoehe - 1 do begin { zeile }
      f.Read ( zeile [ 0 ], kopf.Breite * 3 );
      for k := 0 to 2 do
        for j := 0 to kopf.Breite - 1 do
          TexturArray[i * kopf.Breite*3 + j * 3 + k] := zeile[j * 3 + (2 - k)];
    end
  else
    for i := kopf.Hoehe - 1 downto 0 do begin { zeile }
      f.Read ( zeile [ 0 ], kopf.Breite * 3 );
      for k := 0 to 2 do
        for j := 0 to kopf.Breite - 1 do
          TexturArray[i * kopf.Breite*3 + j * 3 + k] := zeile[j * 3 + (2 - k)];
          { TexturArray[i * kopf.Breite*3 + j] := zeile[j]; }
    end;

  f.Done;
  ReadTGAFile := True;
end;


function SaveScreen2TGA;
var
  i: Integer;
  n: Byte;

begin
  { #define DebugFile }
  #ifdef DebugFile
    Assign(d, 'tgaunit2.dbg');
    Rewrite(d);
  #endif

  if w > MaxWidth then w := MaxWidth;
  Name := Name + '.tga';
  f.Init ( Name, stCreate );
  #ifdef DebugFile
    Writeln(d, 'Rewrite ok');
  #endif
  (*
  if IOResult <> 0 then begin
    SaveScreen2TGA := False;
    return;
  end;
  *)

  FillChar ( kopf, SizeOf ( kopf ), 0 );

  kopf.colormap := 0;   { keine RGB-Tabelle }
  kopf.BildTyp  := 2;   { True-Color        }
  kopf.BitsPerPixel := 24;
  kopf.Unknown  := 0;
  kopf.SubType  := $20; { Top - Down    }
  kopf.SubType  := 0;   { Down - Top    }
  kopf.breite   := w;
  kopf.hoehe    := h;

  { write TGAHeader }
  {$W-}
  #ifdef DebugFile
    Writeln(d, 'w         :', w:0);
    Writeln(d, 'h         :', h:0);
  #endif
  f.Write ( kopf2 [ 1 ], 18  );
  {$W+}
  #ifdef DebugFile
    Writeln(d, 'Kopf geschrieben');
  #endif

  for i := 0 to h*w-1 do begin { zeile }
    { Es scheint ein Bug in GPC zu sein }
    { #255 kann nicht in ein File of Byte geschrieben werden!? }
    {
      zeile[j*3+2] := GrRGBcolorRed(LongPixel);
      zeile[j*3+1] := GrRGBcolorGreen(LongPixel);
      zeile[j*3+0] := GrRGBcolorBlue(LongPixel);
    }
    { Blue }
    n := Min(254, PixelArray[i * 3 + 2]);
    f.Write ( n, 1 );
    { Green }
    n := Min(254, PixelArray[i * 3 + 1]);
    f.Write ( n, 1 );
    { Red }
    n := Min(254, PixelArray[i * 3 + 0]);
    f.Write ( n, 1 );
    {$W-}
    (*
    if (IOResult <> 0) or (Result <> w*3) then begin
      SaveScreen2TGA := False;
      return;
    end;
    *)
    {$W+}
  end;
  f.Done;
  #ifdef DebugFile
    Writeln(d, 'TGA-File geschlossen');
    Close(d);
  #endif
  (*
  if IOResult <> 0 then begin
      SaveScreen2TGA := False;
    return;
  end;
  *)
  SaveScreen2TGA := True;
end;

function RealStr(Zahl:Real):String255;
var
  Zeile : String255;
begin
  WriteStr(Zeile, Zahl:30:15);
  while (Zeile[1] in [' ', '0']) and (Zeile[2] <> '.') do
    Zeile := Zeile[2..length(Zeile)];
  while Zeile[length(Zeile)] = '0' do
    Zeile := Zeile[1..length(Zeile)-1];
  if Zeile[length(Zeile)] = '.' then
    Zeile := Zeile[1..length(Zeile)-1];
  RealStr := Zeile
end;

end.


